/*==================================================================
                     	+------------+
 			|  hlsdef.h  |
                     	+------------+

 This include file is containing all numeric constants and
 structure definitions for the ELETTRA high level software with
 extensive comments. For further explanations on the data structure 
 see the note ST/M-TN-93-?

 First official version. All 0.x versions are outdated !
 New: sextupole_record replaced with magnet_record. PM_type changed
 to SRPM_type. New subtypes for bends added. lumped corrector type
 added.
 Beta 1.0				(Mark Plesko 28.10.92)
 
 machine_file and field constants added. parameter_record: all floats
 set to double, i1 and i2 set to short (see p.Michelini), in.type
 deleted, sc.p_low and sc.p_high changed to sc.b_low and sc.b_high.
 scraper_record: plane deleted. RF_type and special_type added.
 Beta 1.1				(M.P. 10.11.92)
 
 calibration_record changed to contain also reverse calibration
 coefficients (for strength to field conversion), called cr.
 Transfer matrix dimensions changed from 2x2 to 3x3. Constant
 reversed_polarity added, to describe bending magnets with reversed
 bending angle.
 Beta 1.2				(M.P. 15.12.92)
 
 constants machine_file and field deleted.
 Beta 1.3				(M.P. 4.2.93)
 
 constants describing the condition changed to upper case and the
 condition APPLY added, which indicates whether a corrector is used
 for the application of an orbit change. The type BPM_array added
 which is an array to accomodate the single BPM readings (TL and SR).
 Beta 1.4				(M.P. 9.2.93)
 
 The variables port and sid are transferred from the access_record 
 to the host_record, to avoid multiple opening of the same RPC channel.
 The constant AL_channel is added.
 2.0					(M.P. 15.4.93)
 
 Further RPC related changes: host_record gets variable service to
 store the service it is related to. Note that from this version on,
 there is an additional include file hlsrpc.h which contains the
 definitions related to the RPC handling used in functions.c .
 2.1					(M.P. 11.5.93)
 
 The parameter_record for the scraper gets extended to include N_low
 and N_high, the min and max number of steps, respectively. They are
 stored as double (!), since the RPC accepts doubles.
 2.2					(M.P. 22.5.93)
 
 AL_fs increased to accomodate the strT+S file used for precommissioning,
 which contains 15 FS.
 2.3					(M.P. 21.7.93)
 
 Change in parameter_record: I_start in struct ps added, which defines
 the starting current for switching on SR power supplies.
 2.4					(M.P. 16.9.93)
 
 Added three define variables for setting the DCCT read mode.
 2.5					(M.P. 7.10.93)
 
 Added the variable readback in all element records, where the
 reading makes sense in addition to the original variable, where
 the readback was stored before. Now the original variable contains
 the setting. The affected records are:
 ID_record, septum_record, corrector_record, kicker_record, 
 tag_record, family_record.

 3.0					(M.P. 10.11.93)
 
 Added status FB_ON between GOOD and BAD. This will be used for
 the correctors, which are attached to the local feedback.
 3.1					(M.P. 24.10.94)
 
 Added cavity_record for the cavities and modified ID_record to allow
 the treatment of the EEW.
 3.2					(L.T. 9/10/98 )

 Added definition for a chicane corrector and for IDs which are ; Sazaki
 and modulator types. 
 3.3					(L.T. 14/9/99 )
 
 Added the 8th corrector in section 6 --> now AL_co = 83
                                        (F.I. 31-jul-2012)
 
 Added the 8th corrector in section 11 --> now AL_co = 84
                                        (F.I. 08-jul-2013) 
					
 Added the 8th corrector in section 7 --> now AL_co = 85
                                        (F.I. 25-sep-2013) 
					
 Added the 8th corrector in section 9 --> now AL_co = 86
                                        (F.I. 09-dec-2014) 

 Added the 8th corrector in section 10 --> now AL_co = 87
                                        (F.I. 18-jul-2016) 
 ==================================================================*/

#ifndef HLSDEF_DEFINED
#define HLSDEF_DEFINED
/*==================================================================
 Constants used throughout the programs (like PARAMETER in FORTRAN):
 The values of the constants can not be modified !

 Use only the names of constants - never type explicit numbers !
===================================================================*/

/* constants referring to the structure: TL - transfer line,
                                         SR - storage ring */
                                         
#define TL		1
#define SR		2


/* constants refering to the type of the elements: */

#define drift_type	1
#define bend_type	2
#define quad_type	3
#define ID_type		4
#define septum_type	5

#define thin_type	10

#define sextupole_type	11
#define corrector_type	12
#define kicker_type	13
#define scraper_type	14
#define FS_type		15
#define RF_type		16
#define cavity_type	17
#define chicane_type	18

#define passive_type	20

#define SRPM_type	21
#define BPM_type	22
#define CM_type		23
#define special_type	24

#define tag_type	30

/* constants refering to the plane and to the class of
   correctors (there are separate horizontal and vertical
   ones in the transfer line and lumped in the storage ring. */

#define horizontal	0
#define vertical	      1
#define lumped		2

/* constants refering to the class of the bend. At
   ELETTRA we have only the first three classes (com
   stands for combined function bending magnet), but
   theoreticians might want to slice bends into edges
   and several sectors. Bends, which have reversed polarity
   whith respect to what is given in their parameter_record,
   have the class number aumented by reverse_polarity. */

#define hor_rectangle	0
#define ver_rectangle	1
#define com_rectangle	2
#define hor_sector	3
#define ver_sector	4
#define com_sector	5
#define hor_edge	6
#define ver_edge	7

#define reversed_polarity 10

/* constants describing the state of monitors and correctors:
   bad means that it doesn't work properly, dead means that it
   doesn't work at all */
   
#define UNKNOWN		0
#define APPLY		1
#define GOOD		2
#define FB_ON		3
#define BAD		      4
#define DEAD		5


/* constants identifying the type of insertion device: */
   
#define REC		      1    /* Plane permanent magnet ID */
#define hybrid		2    /* Plane hybrid ID */
#define EEW             3    /* Elyptical Electromagnetic Wiggler */
#define modulator       4    /* Plane phase modulation magnet */
#define sazaki          5    /* Helical ID Sazaki type */
#define apu             6    /* apu ID type */


/* constants identifying the mode of the DCCT: current mode,
   lifetime mode and filling rate mode */
   
#define DCCT_current_mode	0
#define DCCT_lifetime_mode	1
#define DCCT_fillrate_mode	2


/* constants used for the definition of array lengths within
   structure definitions (AL stands for array length) */

#define AL_str6		6
#define AL_str13	13
#define AL_str32	32
#define AL_BPM_rpc	12
   
#define	AL_member	24
#define AL_calibration	5
#define AL_kid_calibration	100
#define AL_channel	50 /* the number of possible host/port combinations */


/* define expected array lengths */

#define AL_ge	1400
#define AL_fa	42	/* 12*3Q + QF + QD + 3*S + bend */
#define AL_dr	800
#define AL_be	24	/* increase if rectangle -> edge,sector,edge */
#define AL_qu	108
#define AL_id	34
#define AL_sp	2
#define AL_se	72
#define AL_co	88   /* 82 + the 8th corrector in section 6, 11, 7, 9, 10 and 8 by F.I. 19-apr-2017 */
/* #define AL_co	94  */
#define AL_ki	4
#define AL_sc	3
#define AL_fs	15
#define AL_pr	1
#define AL_po	96	/* extend if BPMs and FS are used together in SR */
#define AL_cm	3
#define AL_ta	20
#define AL_cav	 4
#define AL_chi	 2



/*============================================================== 
   type definitions of structures:
   Note that structures are NOT variables. They only define
   the type of variables.

  structures used in other structure definitions come first:
===============================================================*/


/* the type for the family or member and family_member strings */

typedef char	str6[AL_str6+1] ;	/* allow one more character for '\0' */
typedef char 	str13[AL_str13+1] ; 


/* the type for the host string */

typedef char 	str32[AL_str32+1] ;

/* the type to accomodate the single BPM readings */

typedef double	BPM_array[AL_BPM_rpc] ;


/* the structure which keeps calibration coefficients c (for any type
   of calibrated data) and the reverse calibration coeficients "cr" */

typedef	struct {
	int			Nc, Ncr;
	double			c[AL_calibration], cr[AL_calibration] ;
} calibration_record ;

/* the structure which keeps the roll-off calibration coefficients   */

typedef	struct {
	int			Nc;
	double			gap[AL_kid_calibration] ;
	double			phase[AL_kid_calibration] ;
	double			kx1[AL_kid_calibration] ;
	double			ky1[AL_kid_calibration] ;
	double			kx[AL_kid_calibration] ;
	double			ky[AL_kid_calibration] ;
} kid_calibration_record ;


/* the structure which keeps the host/port pair and the service.
   The variable sid is the RPC service identifier, to be filled at
   run-time by the command sid = import_xxx (host,port) or similar; 
   service keeps the code of the service xxx. */

typedef struct {
	str32			host ;
	int			port ;
	/*---------------------------------*/
	int			sid ;
	int			service ;
} host_record ;


/* the structure which keeps constant parameters such as low and high
   limits of variable parameters, slow and fast ramping speeds for
   power supplies or basic parameters of insertion devices and
   others.
   Note on double vs. float: all limits are of type double for
   compatibility, although their initial precision might be of type float.
*/

typedef union {
	struct { 
		double 	I_low ;		/* I - current */
		double 	I_high ;
		double	I_start ;	/* starting current for SR PS */
		double 	rmp_s ;		/* rmp - ramping speed */
		double	rmp_f ;		/* s - slow, f - fast */
		short	i1 ;		/* ramping data */
		short	i2 ;
		double	length ;	/* theoretical magnet length */
		double	angle ;		/* theoretical bending angle */
	} ps ;	/* ps stands for power supply parameter record */
	struct { 
		double	gh_low ;	/* gh - hor gap */
		double	gh_high ;		
		double	g_low ;		/* g - vert gap */
		double	g_high ;
		int	type ;
		int	Np ;		/* number of periods */
		double	kx ;	/* horizontal transverse coeff of vert field*/
		double	period ;	/* length of one period */
		double	kyh ;	/* vertical transverse coeff of hor field */
	} id ;	/* id stands for insertion device parameter record */
	struct { 
		double	temp_low ;	/* gh - hor gap */
		double	temp_high ;		
		double	voltage_low ;		/* g - vert gap */
		double	voltage_high ;
	} cav ;	/* cav stands for cavity parameter record */
	struct {
		double	V_low ;		/* V - voltage */
		double	V_high ;
		double	t_peak ;	/* t - pulse time */
		double	t_full ;
		double	length ;
		double	angle ;		/* angle of septum */
		double	d_max ;		/* d - delay */
	} in ;	/* in stands for injection elements parameter record */
	struct {
		double	V_low ;		/* V - voltage */
		double	V_high ;
		double	p_low ;		/* p - phase */
		double	p_high ;
		double	f_low ;		/* f - frequency */
		double	f_high ;
	} rf ;	/* rf stands for RF cavities parameter record */
	struct {
		double	b_low ;		/* b - blade position */
		double	b_high ;
		double	s_low ;		/* s - slit size */
		double	s_high ;
		double	N_low ;		/* N - number of steps */
		double	N_high ;
		int	type ;
	} sc ;	/* sc stands for SCraper parameter record */
} parameter_record ;


/* the structure which contains informations of how to access a
   point via the control system. It also contains pointers to other
   important constant data for a given element. 
   family and member are stored separately, because this is the syntax
   for RPC calls (see note by Claudio Scafuri). Each device which is
   accessed may have associated one parameter_record (pointed to by
   pPar) and two calibration_records (pointed to by pCal1 and pCal2). */
		
typedef struct {
	str6			family ;
	str6			member ;
	host_record		*pHost ;
	parameter_record	*pPar ;
	calibration_record	*pCal1, *pCal2 ;
} access_record ;


/* the 3x3 transfer matrix structure for both planes - it is calculated 
   by the function 'update_twiss_functions'. The identifier matrix can
   be used to declare or define two by two matrices, i.e. transfer
   matrices of one plane. */

typedef double 			matrix[3][3] ;	/* the matrix in one plane */
   
typedef struct {
	matrix			m[2] ;		/* array of two matrices */
} matrix_2D ;


/* the structure of Twiss and related variables, which are
   the result of the call to the routine update_twiss_functions:
   the phase is calculated in radians and starts at the first
   element. The values are given at the beginning of each element.  */
   
typedef struct {
	double			beta[2], alpha[2], phase[2],
				D[2], Dprime[2] ;
} twiss_record ;


/*===============================================================
  the main structures for the application programmers:
  In the structure definition first the fixed variables are
  listed. The variables which correspond to a machine setting
  are at the end of the structure definition.
===============================================================*/


/*.............................................................*/
/* pre-declaration of types used in cyclic type references     */

typedef struct general_record	general_record ;

typedef struct family_record	family_record ;

/*.............................................................*/


/* the structure for the drift elements:
   This structure and the following ones (magnet, corrector and simple)
   start with the variables back_index and pGen. back_index is the
   serial number of the element in the general_record list. pGen
   is a pointer pointing directly to the element's general record.
   
   The length (in meters) is the length from the theoretical
   lattice, not the physical length between two element */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	double			length ;
} drift_record ;


/* the structure for magnetic elements:
   this structure is used for bending, quadrupole and sextupole
   magnets. The variable pFam is a pointer to the array of the
   family records (see family_record below), where additional
   information for all magnets of the same family is stored.
   The pointer to the parameter record is duplicated here for 
   convenience (so use pPar instead of pFam->pAcc->pPar). This
   is a feature only for magnets and IDs.
   The pointers pCalB and pCalL are stored here and not in the access_record,
   where there is also space for them, to allow individual
   magnets having different sets of calibration data (requested by
   Albin Wrulich). For sextupoles, only pCalB is used and it
   points to calibration for the integrated sextupole strength !
   sub_type is used to denote the type of bend (see the bend
   type constants in the beginning of this file) or to flag whether
   a quad is used for the conpensation of ID effects (on request of
   Lidia Tosi).
   The variable data for individual magnets are strength ( which
   means the index n for bends, k for quads and m for sextupoles)
   and magnetic_length, which is the actual magnetic length
   obtained from the calibration curve. */

typedef struct {
	int			back_index ;
	general_record		*pGen ;
	family_record		*pFam ;
	parameter_record	*pPar ;
	calibration_record	*pCalB ; /* ptr to field calibration */
	calibration_record	*pCalL ; /* ptr to mag. length cal.  */
	int			sub_type ;
	/*---------------------------------*/
	double			strength ;
	double			magnetic_length ;
} magnet_record ;


/* the structure for insertion devices:
   The constant data are in the parameter record. It could be
   accessed via the access_record, but the pointer is duplicated
   here for the convenience of the programmer.  This is a feature 
   only for bends, quads and IDs.
   The variable data are closed (TRUE if closed, FALSE if open),
   the gap in mm, the inverse bending radius "curvature" and k_x.
   pAcc[0] is the pointer to the access record for the vertical field.
   pAcc[1] is the pinter to the access record for the horizontal field
   ( set to NULL for plane devices ) */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc[2] ;
	parameter_record	      *pPar ;
	kid_calibration_record  *pKal;
	/*--------------------------------*/
	int			      closed ; /* no Boolean type in C */
	double			readback, gap ;
	double                  readback_h, gap_h  ;
	double                  phase;
	double			curvature ;
	double			curvature_h ;
	double			k_x, k_y ;
	double			kh_x, kh_y ;	
} ID_record ;


/* the structure for septa:
   This is used for simple data - it contains just the basic data to 
   work with. Other injection related data must be supplied by the
   application programmer, if needed.
   */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	double			readback, voltage ;
	double			curvature ;
} septum_record ;


/* the structure for correctors:
   the correctors have the access data, the condition, the current 
   and the theta (in mrad) in arrays of two, for the horizontal and 
   vertical plane. The variable condition is to be filled by the
   application programmer, if he/she wants to make use of it as a
   flag for the condition of the corrector (in that case it is 
   recommended to use the constants declared in the beginning of
   this file).
   Use the constants 'horizontal' and 'vertical' defined in the beginning
   of this include file when refering to a specific plane. When
   using a do loop over both planes, use these constants to define
   the loop initial and final value. */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc[2] ;
	int			plane ;
	/*-----------------------------------*/
	int			condition[2] ;
	double			readback[2], current[2] ;
	double			theta[2] ;
} corrector_record ;


/* the simple structure for kickers:
   This is used for simple data - it contains just the basic data to 
   work with. Other injection related data must be supplied and stored
   by the application programmer. */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	double			readback, voltage ;
	double			theta ;
} kicker_record ;


/* the simple structure for scrapers:
   This is used for normal users of scrapers - it contains just the
   basic data to work with, no special settings. 
   The values position and slit_size are measured in mm. As they are
   known directly, no conversion is necessary. */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	double			position ;
	double			slit_size ;
} scraper_record ;


/* the structure for monitors:
   the SRPM, FS and TL BPMs are read individually, via RPC addresses given
   in the access record. The SR BPMs are read in one go, so pAcc is
   NULL for those.
   The condition is denoting the state of the monitor. It is up
   to the application programmer to set its value to one of the
   constants defined at the beginning of this file.
   Both beam_position and rms are two dimensional - use the 
   constants 'horizontal' and 'vertical' to access the data. For
   BPMs rms is the root mean square over several consecutive
   measurements (if they were requested to the server). For FS and SRPM
   rms stands for the width of the electron beam. */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	int			condition ;
	double			beam_position[2] ;
	double			rms[2] ;
} monitor_record ;


/* the structure for current monitors:
   This is the simplest record, as current monitors only return one
   value - the beam current. There is no need for calibration or for
   parameters. */

typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	double			beam_current ;
} CM_record ;

/* the structure for cavities: 
pAcc[0] is the access record pointer for the temperature
pAcc[1] is the one for the voltage */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc[2] ;
	parameter_record        *pPar ;
	/*-----------------------------------*/
	double			readback, temperature ;
	double			voltage ;
} cavity_record ;

/* the structure for tags:
   This is used for simple data structures - it contains just the
   basic data to work with. For tags which are related to real, accessible
   objects, the application programmer must set the pointer pAcc to the 
   proper content. Originally pAcc is set to NULL.
   For tags which do not point to real objects, raw_value 
   and converted_value can still be used, but it is the responsibility 
   of the application programmer to fill them with the proper data. */
   
typedef struct {
	int			back_index ;
	general_record		*pGen ;
	access_record		*pAcc ;
	/*-----------------------------------*/
	double			readback, raw_value ;
	double			converted_value ;
} tag_record ;


/*...............................................................

   the predeclared structures are defined here directly as 
   structures.

...............................................................*/


/* the structure describing magnetic familes:
   a family is a group of identical magnets powered by the
   same power supply. The data common to all magnets in a
   family are stored here: the pointer to the access record, 
   a list of all members of the family and the variable current,
   which is the PS current in Amps. */
   
struct family_record {
	access_record		*pAcc ;
	int			N_member ;
	magnet_record		*pMember[AL_member] ;
	/*------------------------------------------*/
	double			readback, current ;
} ;


/* the basic structure for all elements:
   The name is used only for easier identification of elements. 
   It should be equal to the ELETTRA naming convention and it is 
   arbitrary for drift elements. The section is an integer number 
   giving the ELETTRA section in which the element is. The type is 
   one of the constants in the beginning of this include file (1-30). 
   The position is measured in meters starting from the beginning of 
   the first element up to the beginning of the current element.
   The index gives the element number in the array of records for
   the given type. The pointers p.dr, p.be, etc. point to the type
   record directly. This redundancy is made for the convenience of
   the application programmer.  
   transfer and twiss are pointers to the transfer matrices and the
   twiss record for the element. For zero length elements (type >= thin_type),
   transfer points to a unit matrix, and twiss points to the twiss record of
   the first non-zero length element after the zero length element.
   The transfer_matrix is set to the proper value for drift elements
   (IDs and magnets are also treated as drifts in this case). 
   The Twiss parameters are given at the beginning of the element to be 
   consistent with the definition of the twiss values. */

	
struct general_record {
	str13			name ;
	int			section ;
	int			type ;
	double			position ;
	int			index ;
	union {		/* the pointers to the type records */
		drift_record		*dr ;		/* drift */
		magnet_record		*be,*qu;	/* bend,quad */
		ID_record		      *id ;		/* ID */
		septum_record		*sp ;		/* septum */
		magnet_record		*se ;		/* sextupole */
		corrector_record	      *co ;		/* corrector */
		kicker_record		*ki ;		/* kicker */
		scraper_record		*sc ;		/* scraper */
		monitor_record		*fs,*pr,*po ;	/*FS,PRofile,POsition*/
		CM_record		      *cm ;		/* curr.mon. */
		cavity_record		*cav ;		/* cavity */
		tag_record		      *ta ;		/* tag */
		corrector_record		*chi ;		/* chicane */
		void			      *generic ; /* used by system only */
	} p ;
	/*--------------------------------------------------------*/
	matrix_2D		*transfer ;
	twiss_record		*twiss ;
} ;


/*================================================================
  *********************** end of hlsdef.h **********************
================================================================*/
#endif 
