/*==================================================================
                     	+----------------+
 			|    hlsfun.h    |
                     	+----------------+

 This include file is containing all function definitions from the
 hls libraries for the ELETTRA high level software. For further 
 explanations on the data structure see the note ST/M-TN-92-?

 Newly created. Input functions and some conversion functions
 are defined.				 	(Mark Plesko, 15.9.92)
 
 changed parameter type in update_twiss_functions - now pointer to
 twiss_record is taken instead of record itself.
 Beta 1.1					(M.P. 16.12.92)

 The parameter to input_structure() has been changed to a string (str32)
 which represents the file name of the structure file. input_structure()
 now returns -1 if the file could not be opened and 0 if it was OK.
 input_special() is deleted, as all special elements are read in
 during input_open_files().
 Beta 1.3					(M.P. 18.12.92)

 field_to_strength, strength_to_field, f2s_magnets and f2s_correctors
 function type and argument structure changed.
 Beta 1.4					(M.P. 4.2.93)

 many new functions added, list now complete.
 Beta 1.5					(M.P. 15.2.93)

 Changed init_model_from_xxx such that no twiss_record is passed.
 New get_single_FS() now according to the specs by M. Ferianis.
 Modes "RE" and "WE" for RPC now supported.
 Added get_single_voltage to read the septum and kicker high voltages.
 Changed read_magnets_from_xxx and f2s_magnets to include also the
 septa, such that all the magnetic elements which contribute to the
 optics are handled in one routine. Deleted the now obsolete routine
 read_septa_from_file.
 New plot_twiss(), which allows to plot the beta functions.
 Beta 1.6					(M.P. 23.2.93)

 New version of plot_twiss(), which returns also the dispersion
 functions.
 Beta 1.7					(M.P. 24.3.93)

 The routine cleanup_rpc() to allow for performing free_lpc() is added.
 A new RPC return value scheme was adopted: 0: no error; -1: rpc error;
 -2: import error; -3: rpc type nonexistent; -4: import after cleanup_rpc 
 not allowed. Other errors will be introduced later.
 2.0						(M.P. 15.4.93)
 
 Changed the parameter list for get_single_current.
 This include file is now very outdated. But I don't have really time
 to make it complete. Later...
 2.1						(M.P. 11.11.93)
 ==================================================================*/

/*******************************************************************/
/* data input routines: they read the ORACLE databases and build
   up the global data structure. */
   
extern void	input_open_files() ;
		/* no parameters */
/* opens ORACLE files, initializes some internal data, reads special
   elements */
   
extern int	input_structure( str32 ) ;
		/* parameter name of structure file */
/* reads structure file and fills global data - either storage ring
   or transfer line, depending on the structure given in the file. The
   functions returns -1 if the file could not be opened and 0 if all
   went well. */

extern void	input_special( const int, str13, access_record ** ) ;
		/* const int: type of the access record (special_type usually),
		   str13: a key naming the special element,
		   access_record *: a pointer to the access record
		                    of this special element */
/* reads special elements data from the ORACLE database. Generally
   not needed, as all the 'normal' special elements are read in
   the routine input_open_files. Used only by CJB for now. Returns
   -1 if special element not found, 0 otherwise. */
   
extern void	input_close_files() ;
		/* no parameters */
/* close ORACLE files, deallocates some internal data */

extern void	cleanup_rpc () ;
		/* no parameters */
/* close all RPC connections. After having called this routine, no
   further remote procedure calls may be done. This routine should
   be called right before exiting a program, to provide a clean 
   termination of RPC allocations. */


/*******************************************************************/
/* single point read/write routines: they refer to one variable, either
   from the machine file or from the field via RPC. They all return
   the value 0 if the read was successful or the negative RPC error
   code otherwise. */

extern int	wait_BPM_average () ;
		/* no parameters */
/* waits the time required for a new average of the BPM measurements.
   It makes a rpc to find this time and returns the error of this rpc. */

extern int	get_single_BPM ( monitor_record *, double *, double * ) ;
		/* monitor_record *: the pointer to the BPM,
		   BPM_array: the array returned by the single BPM
		              read server (different length for TL and SR) */

extern int	get_single_FS ( monitor_record *, double *, double *,
						  double *, double * ) ;
		/* monitor_record *: the pointer to the FS,
		   double: horizontal beam position
		   double: vertical beam position
		   double: horizontal beam size
		   double: vertical beam size      */
/* pushes the FS into the chamber, waits for the measurement and pulls
   the FS back out. This may take some time. Returns a negative value
   at the first RPC error it encounters. */

extern int	get_single_current ( access_record *, double *, double * ) ;
		/* access_record *: the pointer to the PS access data,
		   double: current */
/* reads any single PS and returns its current. */

extern int	get_single_voltage ( access_record *, double *, double * ) ;
		/* access_record *: the pointer to the PS access data,
		   double: voltage */
/* reads any single kicker or septum and returns its high voltage. */

extern int	read_current_from_file ( str32, access_record *, double *, double * ) ;
		/* str32: the name of the machine file,
		   access_record *: the pointer to the PS access data,
		   double: current */
/* reads any single PS current from the machine file. -1 is returned if
   the file is not found, -2 if the PS is not found. */

extern int	set_current_fast ( access_record *, double ) ;
		/* access_record *: the pointer to the PS access data,
		   double: current to be set */
/* sets the current of a power supply using the lpc_WTIPOL routine
   (fast ramp). For the time being there is no provision for the
   programmer to decide either the fast (rmp_f) or slower (rmp_s)
   ramping slope. This is done in the routine and is thus apllied to
   all PS equally. If the RPC was successful, an integer giving the
   expected ramp time in milliseconds is returned. It is up to the programmer
   to wait for the finish of the ramp. */
   
extern int	set_current_slow ( access_record *, double ) ;
		/* access_record *: the pointer to the PS access data,
		   double: current to be set */
/* sets the current of a power supply using the lpc_WR routine
   (bit by bit ramp). This is very slooooooow */


/*******************************************************************/
/* field data read routines: they read field variables in groups,
   so that the programmer doesn't have to write the loops himself. All
   the read in data are placed directly into the data bush. The
   functions return 0 if they were successful, a negative value otherwise.
   For reading the field the first RPC error that was encountered is
   reported, because after that the function returns immediately. */
   
extern int	read_TL_twiss ( twiss_record * ) ;
		/* parameter the twiss record which contains the
		   initial twiss parameters for the TL. */
/* reads the file where the measured linac exit conditions are stored.
   It is the responsibility of the HLS manager to keep this file updated.
   Returns -1 if file not found. */

extern int	read_energy_from_field ( const int ) ;
		/* parameter either SR or TL */
/* reads the current from a bending magnet PS (either from the storage
   ring - case SR, or from the transfer line - case TL), converts
   it to the energy in GeV and stores it in the global structure energy.
   Returns a -1 if another structure than TL or SR was requested and -2
   if the calibration record is corrupted. */
   
extern int	read_energy_from_file ( str32 ) ;
		/* parameter machine file name */
/* as above, but reads the energy from the given machine file. Returns
   -1 if file not found. */

extern int	read_magnets_from_field () ;
		/* no parameters */
/* reads the currents for all magnets (bend, quad, sextupole) and the
   voltages for all septa (if any) from the global lists (fa[] and sp[] 
   arrays) and puts the currents ad voltages, respectively, in the proper
   variables. 0 is returned if read was succesful, a negative number 
   if not (probably equal to the RPC error from the faulty read). */
   
extern int	read_magnets_from_file ( str32 ) ;
		/* parameter machine file name */
/* as above, but reads the currents and voltages from the given machine 
   file. Returns -1 if file not found and -2 * #(not found magnets) if 
   some magnets were not found in the file. */

extern int	read_IDgaps_from_field () ;
		/* no parameters */
/* reads the gaps for all IDs listed in the global lists (id[] array) and 
   puts the gaps in the proper variables. 0 is returned if read was 
   succesful, a negative number (probably equal to the RPC error from the
   faulty read) otherwise. */

extern int	read_IDgaps_from_file ( str32 ) ;
		/* parameter machine file name */
/* as above, but reads the gaps from the given machine file. Returns
   -1 if file not found and -2 * #(not found IDs) if some of them
   were not found in the file. The status of IDs that were not found
   is set to closed = 0 (i.e. open, not active), the IDs found get
   closed = 1. */

extern int	read_correctors_from_field () ;
		/* no parameters */
/* reads the currents for all correctors (horizontal and vertical) in the
   global lists (co[] array) and puts the currents in the proper
   variables. 0 is returned if read was succesful, a negative number
   (probably equal to the RPC error from the faulty read) otherwise. */

extern int	read_correctors_from_file ( str32 ) ;
		/* parameter machine file name */
/* as above, but reads the currents from the given machine file. Returns
   -1 if file not found and -2 * #(not found correctors) if some of them
   were not found in the file. */

extern int	read_FS_TL_from_field ( int *) ;
		/* int: the index of the first screen where an error occured */
/* calls the function get_single_FS for all FS in the global data bush and
   places the read in values in the data bush. If all screens were read 
   without errors, then the index of the first erroneous screen is N_FS. */

extern int	read_BPM_from_field ( const int ) ;
		/* parameter either SR or TL */
/* reads the beam position at all BPMs in the transfer line or the 
   storage ring, depending on the parameter given, and fills the data bush
   variables. 0 is returned if read was succesful, a negative number
   (probably equal to the RPC error from the faulty read) otherwise. The
   value -1 is returned if a system other than TL and SR was chosen.
   The value -2 is returned and a warning printed to stdout, if no free
   memory could be allocated for internal needs. */

extern int	read_BPM_SR_from_file ( str32 ) ;
		/* parameter machine file name */
/* as above, but reads the positions from the given machine file. 
   Returns -1 if file not found, -2 if there was no BPM info in the
   machine file. */


/*******************************************************************/
/* field data set routines: they set field variables in groups,
   so that the programmer doesn't have to write the loops himself. All
   the data to be set are taken from the data directly. The
   functions return 0 if they were successful, a negative value otherwise,
   usually the first RPC error that was encountered. */

extern int	set_correctors_fast ( const int, int * ) ;
		/* const int: either horizontal or vertical */
/* sets all the correctors in the given plane to the currents stored
   in the data bush, but only those correctors, whose condition is
   APPLY. After the set the condition is restored to GOOD (only those
   correctors should be used, which have condition GOOD). Even if
   a RPC error is encountered, the procedure continues setting the other
   correctors, however, the condition of the faulty corrector is left
   to be APPLY, so the programmer can track it down. The routine returns
   the sum of encountered RPC errors.
   The RPC used is lpc_WTIPOL, i.e. the fast ramping routine. */

extern int	set_correctors_slow ( const int, int * ) ;
		/* const int: either horizontal or vertical */
/* as above, only that the setting in the bit by bit mode (lpc_WR). */


/*******************************************************************/
/* conversion routines: they convert from field values (current,
   voltage, etc.) to energy normalized values (index, k, etc.) and
   vice versa. */
   
extern int	field_to_strength (calibration_record *, double, double *) ;
		/* pointer to the calibration data for the conversion,
		   double: field value to be converted,
		   double:  normalized strength      */
/* converts a field value to a normalized strength. The energy is taken
   from the structure energy, so a read_energy routine MUST be called
   before caling this routine. The value 0 is returned if the conversion
   was succesful, -1 if the calibration data is corrupted and -2 if
   the energy was not defined previously. */
   
extern int	strength_to_field (calibration_record *, double, double *) ;
		/* pointer to the calibration data for the conversion,
		   double: normalized strength to be converted,
		   double: field value        */
/* As above, only in the opposite direction, i.e. a normalized strength 
   is converted to a field value. */

extern int	IDgap_to_strength (ID_record *, double, double *, double *) ;
		/* pointer to the ID record,
		   double: ID gap to be converted,
		   double: resulting k_x of the ID
		   double: resulting curvature of the ID field */
/* As field_to_strength, only for IDs. The error messages are the same */ 
  
extern int	f2s_magnets () ;
/* converts the currents of all magnets into normalized strenghts
   (no conversion for magnetic length!) and the voltages of all septa
   (if any) into their curvature and stores them in the global data
   structure. The currents and voltges MUST have been read in before, either
   individually or by calling read_magnets ! The return value is 0 if the
   conversion was successful, the sum of the individual errors otherwise. */

extern int	f2s_IDs () ;
/* converts the gaps of all IDs into the proper k_x and curvature values
   and stores them in the global data structure. The gaps MUST have 
   been read in before, either individually or by calling read_IDgaps !
   The return value is 0 if the conversion was successful, the sum of 
   the individual errors otherwise. */

extern int	f2s_correctors () ;
/* converts the currents of all magnets into normalized strenghts
   and stores them in the global data structure. The currents MUST have 
   been read in before, either individually or by calling read_magnets !
   The return value is 0 if the conversion was successful, the sum of 
   the individual errors otherwise. */

extern int	s2f_correctors () ;
/* as above, only in the opposite direction, i.e. strength to field. */
   

/*******************************************************************/
/* special routines which simplify orbit correction for R. Nagaoka */

extern int	apply_correction ( const int ) ;
		/* const int: either horizontal or vertical */
/* combining s2f_correctors and set_correctors_fast. The error
   reporting is to be discussed with R.N. */

extern int	acquire_correction () ;
		/* no parameters */
/* combining read_correctors_from_field and f2s_correctors. The error
   reporting is to be discussed with R.N. */


/*******************************************************************/
/* optics routines: perform optics calculations and fill or use
   the transfer and twiss structures in the global_records */

extern int	get_transfer_matrix ( general_record *, matrix, matrix ) ;
		/* general_record *: pointer to general element,
		   matrix: horizontal 3x3 transfer matrix,
		   matrix: vertical 3x3 transfer matrix */
/* Courtesy of R. Nagaoka:
   calculates transfer matrix of given element, returns it via the
   argument list and also updates the transfer matrix in the data
   bush. Returns 0 if all was OK, -2 if element type is unknown
   and -3 if element type is bend and its subtype is not known */

extern int	update_twiss_functions ( const int, twiss_record *) ;
		/* const int: either SR or TL,
		   twiss_record*: case TL the initial twiss (input)
		                  case SR the calcuated twiss in the
		                  first element (output) */
/* Courtesy of R. Nagaoka:
   generates the transfer matrices and twiss function values at all
   elements stored in the structure (ge[] array). The first parameter
   defines that either a closed solution (SR) or a non-closed solution
   (TL) is requested. In the first case, the previously allocated 
   twiss_record pointed to by the second parameter is filled with the
   twiss parameters at the starting point of the structure.
   In the latter case, the twiss_record is taken as input containing
   the initial conditions . If a solution was found, the function 
   returns 0, otherwise it returns -1 if no closed solution exists
   (in the case of SR), -2 if and element whose type is unknown was
   encountered and -3 if a bends subtype was not known (see above
   for get_transfer_matrix). */

extern int 	plot_twiss( twiss_record *, int, double, int, double, 
			int, int, double *, double *, double *, double *) ;
/* Courtesy of L. Tosi:
   a copy of the original explanation follows:
   
int plot_twiss( ptrTwiss0, io, so, il, sl, fnom, n, ptr_bh, ptr_bv,
               ptr_dh,ptr_dv)
returns four arrays containing respectively 'n' horizontal beta values in
'ptr_bh' , 'n' vertical ones in 'ptr_bv' , 'n' horizontal dispersion values in
'ptr_dh & 'n' vertical ones in 'ptr_dv' for the plot widget in the uniform mode.
The values are computed from the distance 'so' from the start of 
the element 'io' of the general_record in the data bush to the distance 'sl'
from the start of the element 'il'.
'ptrTwiss0' is a pointer to a twiss_record containing ( please NOTE ) the 
initial twiss parameters at the BEGINNING of element 'io'.
'fnom' is a boolean flag which set to the value TRUE ( alias 1 in C )
will indicate that the optics are the nominal ones, i.e. the insertion
devices are treated as drifts. If 'fnom' is set to the value FALSE, the
linear effects of the insertion devices will be considered.
Please note that the computation is progressive, i.e. it goes from 'io' to
'il', which may be less than 'io'.

/*******************************************************************/
/* utility routines which simplify the work for people who do not
   really care about the details and just want to get their model */

extern void	input_all ( str32 ) ;
		/* parameter name of structure file */
/* combines input_open_files, input_structure and input_close_files. If
   it can not find the structure or any other file, it exits printing
   a message to stdout. */

extern int	init_model_from_field ( const int ) ;
		/* parameter either SR or TL */
/* combines read_energy_from_field, read_magnets_from_field, 
   read_IDgaps_from_field, f2s_magnets, f2s_IDs, read_TL_twiss
   (in case of the parameter being TL) and update_twiss_functions.
   It returns 0 if all was correct, -1 ifthe energy or - in case
   of the TL - the initial twiss functions could not be read,
   -2 if the from_field routines gave errors, -3 if the f2s routines
   gave errors and -4 if update_twiss_functions returned errors. 
   This routine should be used to get the full model from the real
   field data. */

extern int	init_model_from_file ( str32, const int) ;
		/* str32: name of machine file,  
		   const int: either SR or TL */
/* as above, but takes all data from the machine file. */		

/*==================================================================
  ************************* end of hlsfun.h **********************
==================================================================*/
