/*==================================================================
                     	+------------+
 			|  hlsrpc.h  |
                     	+------------+

 This include file is containing all numeric constants and
 structure definitions related to the handling of remote procedure
 calls (RPCs) for the ELETTRA high level software. It is normally
 not used by the programmer, who can call higher level routines
 without referring to actual RPCs.

 First version.
 Beta 1.0				(Mark Plesko 11.5.93)
 
 Variables changed: bpm.x_s to bpm.s_x and bpm.y_s to bpm.s_y, added
 the variable bpm.s, to be consistent with new note by M.Lonza (29.6.93)
 
 1.1					(M.P. 10.8.93)
 
 Added variables for the save/restore (sare) server into the struct
 RPC_list. Added the constant service_sare.
 1.2					(M.P. 26.10.93)
 
 Removed parameter "action" from the parameter list of the cyc server,
 because it is not used by cyc_wtipol. The type of datum changed from
 (void *) to (double *).
 1.3					(M.P. 2.11.93)
 
 Changed type of stat variables in sare service from double to int.
 1.4					(M.P. 27.5.94)
 
 Added new service flsc, to read the FS image (or SRPM image).
 1.5					(M.P. 21.3.95)
 
 Added new service info, to get the general beam info. Array of 
 strings, service_name[], deleted, as it is in rpc.c locally.
 1.6					(M.P. 21.11.95)
 ==================================================================*/

#include <errtype.h>
#include <ncc.h>

/* from include <lpc.h> */

typedef	str6	familyName ;
typedef	str6	memberName ;
typedef	str6	actionName ;
typedef	str6	modeName ;

typedef struct {
	cardinal length;
	str6 *sequence;
} familyArray, memberArray ;

typedef struct {
	cardinal length;
	boolean *sequence;
} booleanArray;

typedef struct {
	cardinal length;
	character *sequence;
} charArray;

typedef struct {
	cardinal length;
	integer *sequence;
}shortArray;

typedef struct {
	cardinal length;
	long_integer *sequence;
} intArray;

typedef struct {
	cardinal length;
	long_floating *sequence;
} realArray;

typedef struct {
	long_floating curr;
	integer iperc1;
	integer iperc2;
	long_floating alpha;
} WTIPOL_data;


 
/* constants referring to the RPC service */
                                         
#define service_lpc		1
#define service_cyc		2
#define service_bpm		3
#define service_bump		4
#define service_inj		5
#define service_sare		6
#define service_flsc		7
#define service_info		8


/* The structure which is used for passing the parameters to 
   different types of RPCs via the same variable (a pointer to
   this structure. On an explanation of the RPCs and the variables
   used see the control group notes by C.Scafuri and M.Lonza.*/
   
typedef union {
	struct { 
		char 		*family ;
		char		*member ;
		char		*action ;
		void		*datum ;
	} lpc ;	
	struct { 
		char 		*family ;
		char		*member ;
		double		*datum ;
	} cyc ;	
	struct { 
		double		*s_x ;
		double		*s_y ;
		double		*x ;
		double		*xrms ;
		double		*y ;
		double		*yrms ;
		double		*s ;
		int		*errn ;
		unsigned int	*time ; /* C.S. 10/11/2005*/
		short		*n_sample ;
	} bpm ;
	struct {
		familyArray	*family ;
		memberArray	*member ;
		realArray	*curr ;
		intArray	*errn ;
		int		steps ;
		int		*busy, *stop, *step ;
		char		*message ;
	} bump ;
	struct { 
		char 		*family ;
		char		*member ;
		char		*action ;
		void		*datum ;
		int		*errn ;
		int		*errt ;
	} inj ;	
	struct { 
		double		*curr ;
		short		*errcurr ;
		double		*curset ;
		short		*errcurset ;
		int		*stat ;
		short		*errstat ;
		boolean		*flag ;
		short		*errn ;
		long		*max_time ;
	} sare ;	
	struct {
		void		*image ;
	} flsc ;
	struct {
		double		*curr ;
		int		*cflag ;
		double		*life ;
		int		*lflag ;
		double		*injrate ;
		int		*injflag ;
		double		*intcurr ;
		int		*intflag ;
		double		*energy ;
		int		*engflag ;
	} info ;
} RPC_list ;
		
