/*==================================================================
                     	+----------------+
 			|    hlsvar.h    |
                     	+----------------+

 This include file is containing all global variables for the 
 ELETTRA high level software with extensive comments. For further 
 explanations on the data structure see the note ST/M-TN-92-?

 First official version. All 0.x versions are outdated !
 New: sextupole_record replaced with magnet_record. N_PM changed
 to N_SRPM.
 Beta 1.0				(Mark Plesko 28.10.92)
 
 A matrix_2D unit matrix is defined.
 Beta 1.1				(M.P. 6.11.92)

 The global variable model_tune[2] is added.  pAccBPM_AvgT and pAcc_BPM[2]
 added. pAccTL and pAccSR in energy changed to pAcc[2]
 Beta 1.2				(M.P. 10.2.93)
 
 The access pointer for the DCCT added (pAccDCCT) and pAcc_BPM[2]
 changed to pAccBPM_SR (because there is no way anymore to read all
 TL BPMs). pAccBPM_AvgT is deleted, because the same information is
 contained in pAccBPM_SR. Note: they are used in the bpm service.
 V 1.3					(M.P. 2.8.93)

 The energy_record gets another variable: readback. This corresponds
 to the energy determined from the read current, while E now corresponds
 to the energy calculated from the PS set values.
 2.0					(M.P. 10.11.93)
 
 Added pAccInfo, the access record of the beam info sever.
 2.1					(M.P. 21.11.95)
 ==================================================================*/

/* first include HLS type definitions */
#ifndef HLSVAR_DEFINED
#define HLSVAR_DEFINED

#include "hlsdef.h"


/* define arrays of pointers to data structures */

extern general_record		*ge[] ;
extern family_record		*fa[] ;
extern drift_record		*dr[] ;
extern magnet_record		*be[] ;
extern magnet_record		*qu[] ;
extern ID_record		      *id[] ;
extern septum_record		*sp[] ;
extern magnet_record		*se[] ;
extern corrector_record		*co[] ;
extern kicker_record		*ki[] ;
extern scraper_record		*sc[] ;
extern monitor_record		*fs[] ;
extern monitor_record		*pr[] ;
extern monitor_record		*po[] ;
extern CM_record		      *cm[] ;
extern cavity_record		*cav[] ;
extern tag_record		      *ta[] ;
extern corrector_record		*chi[] ;


/* define variables used for program flow control */

extern int		N_general;	/* the actual number of all elements */
extern int		N_family;	/* the actual number of mag. families*/
extern int		N_drift;	/* the actual number of drifts */
extern int		N_bend;	/* the actual number of bend magnets */
extern int		N_quad;	/* the actual number of quads */
extern int		N_ID;		/* the actual number of IDs */
extern int		N_septum;	/* the actual number of septa */
extern int		N_sextupole;/* the actual number of sextupoles */
extern int		N_corrector;/* the actual number of correctors */
extern int		N_kicker;	/* the actual number of kickers */
extern int		N_scraper;	/* the actual number of scrapers */
extern int		N_FS;		/* the actual number of Fl.Screens */
extern int		N_SRPM;	/* the actual number of Profile Mon. */
extern int		N_BPM;	/* the actual number of monitors */
extern int		N_CM;		/* the actual number of Curr.Mon. */
extern int		N_cavity;	/* the actual number of cavities */
extern int		N_tag;	/* the actual number of tags */
extern int		N_chicane;	/* the actual number of chicanes */


/* the declaration of variables used for optics calcuations */

extern int		      N_section;	/* the number of sections */
extern double		section_length;	/* the length of one section */
extern double		total_length;	/* the length of the whole structure */
extern double		model_tune[];	/* the tune of the whole structure */

/* the information how to get the energy readings */

extern struct {
	access_record	*pAcc[2] ;
	double		readback, E ;
} energy ;


/* the access pointers to special data (read with input_special in
   the routine input_close_files) */

extern access_record	*pAccBPM_SR, *pAccTune, *pAccDCCT, *pAccRF, *pAccInfo ;


/* the unit matrix in both planes - it is pointed to by ge[]->transfer
   of thin elements. It may be used to initialize other matrices (in
   ANSI C structs may be assigned directly). */
   
extern matrix_2D	unit_matrix ;


/*==================================================================
  ************************* end of hlsvar.h **********************
==================================================================*/
#endif
